#include <stdio.h>

  /* Zadatak 4:
     Napisite funkciju koja prima niz znakova s n elemenata. Funkcija treba
     vratiti prirodni broj k ako i samo ako regularni izraz
	 (ab*)c+(\1)+
     prepoznaje pocetni dio od prvih k znakova zadanog niza, a dulji
     pocetni dio (ako postoji) vise ne prepoznaje. U tom slucaju, kroz
     varijabilni argument treba vratiti broj pojavljivanja znaka 'a' u
     prepoznatom pocetnom dijelu niza. U protivnom, funkcija treba vratiti 0,
     a kroz varijabilni argument ne treba nista vratiti.
  */

  /* Skica rjesenja, prema zadanom regexu:
     Polje pretrazujemo od pocetka ("pocetni komad")
       1. nadji prvu grupaciju \1 koja odgovara (ab*),
       2. provjeri c+ (blok c-ova, mora bar jedan) iza toga,
       3. provjeri tocno ponavljanje \1 (mora bar jednom) iza toga.
     Prepoznati pocetni blok (ako postoji) je na mjestima od 0 do k-1.
     Cijelo vrijeme treba paziti da ne predjemo zadanu duljinu niza n.
     Dozvoljeno je pretpostaviti da je n >= 1 (iako se testira ispod).
  */

int regex(char s[], int n, int* p_broj_a)
{
    int k = 0,        // Inicijalizacija duljine prepoznatog niza na 0.
        broj_a = 0,   // Inicijalizacija broja pojavljivanja znaka 'a'.
        kraj_1 = -1,  // Indeks kraja prve grupacije \1.
        i,            // Indeks za setanje po nizu s.
        j;            // Indeks za setanje po prvoj grupaciji.

        /* Provjera n >= 1, za svaki slucaj (iako nije nuzno). */
    if (n < 1) return k;

        /* Korak 1:
           Nadji prvu grupaciju (ab*), tj. prvi znak mora biti 'a',
           a iza je blok znakova 'b' (ne mora ih biti). */

        /* Ako prvi znak nije a, ne valja => vrati se odmah. */
    if (s[0] != 'a') return k;
        /* Povecaj broj a-ova. */
    ++broj_a;
        /* Idi naprijed sve dok su b-ovi, do kraja niza ili do znaka
           razlicitog od b. */
    for (i = 1; i < n  && s[i] == 'b'; ++i);
        /* Ako su b-ovi sve do kraja, ne valja => vrati se odmah. */
    if (i == n) return k;
        /* Zapamti poziciju zadnjeg znaka b, to je kraj prve grupacije. */
    kraj_1 = i - 1;

        /* Korak 2:
           Provjeri c+ (blok c-ova, mora bar jedan) iza toga. */

        /* Ako prvi sljedeci znak nije c, ne valja => vrati se odmah. */
    if (s[i] != 'c') return k;
        /* Idi naprijed sve dok su c-ovi, do kraja niza ili do znaka
           razlicitog od c. */
    for (++i; i < n  && s[i] == 'c'; ++i);
        /* Ako su c-ovi sve do kraja, ne valja => vrati se odmah. */
    if (i == n) return k;

        /* Korak 3:
           Provjeri tocno ponavljanje \1 (mora bar jednom) iza toga. */

        /* Vanjski do-while ide po cijelim grupacijama oblika \1.
           Koristimo do-while, jer mora biti barem jedna takva jos u nizu.
           Polazna grupacija \1 nalazi se na mjestima s[0], ..., s[kraj_1]. */
    do {
            /* Provjera sljedece pojave grupacije \1 u nizu. Sljedeci znak
               za provjeru je s[i]. Pomak petlje povecava oba indeksa i, j. */
        for (j = 0; j <= kraj_1 && i < n && s[i] == s[j]; ++i, ++j);
            /* Ako je j > kraj_1, nasli smo cijelu novu pojavu grupacije \1.
               Povecaj broj a-ova i postavi novu duljinu prepoznatog dijela.
                 U protivnom, nismo nasli novu grupaciju \1, pa sve ostaje
               kako je bilo prije toga i prekidamo vanjsku petlju. */
        if (j > kraj_1) {
            ++broj_a;
            k = i;     // Duljina je i, na mjestima s[0], ..., s[i-1].
        } else
            break;

        /* Uvjet je while(1), jer izlazimo s break. */
    } while (1);

        /* Vrati izlazne vrijednosti, prvo varijabilni argument. */
    if (k > 0) *p_broj_a = broj_a;
    return k;
}


  /* Dobri primjeri za test:
       abbcababababab    <- ne prepozna nista, kako i treba
     U ostalim promjerima prepozna cijeli niz:
       abbcabbabbabbabb
       abcabab
       acaa
  */

int main(void) {
    char s[] = "abcabab11" ;    // 3. primjer + visak na kraju.
    int n = 9, i;
    int dulj, broj_a;

        /* Ispis polaznog niza. */
    printf("Polazni niz:\n");
    for (i = 0; i < n; ++i)
        printf("%c", s[i]);
    printf("\n\n");

        /* Poziv funkcije. */
    dulj = regex(s, n, &broj_a);

        /* Ispis rezultata. */
    printf("Duljina prepoznatog pocetka = %d\n", dulj);
    if (dulj > 0)
        printf("Broj znakova a = %d\n", broj_a);

    return 0;
}
